@@ -12,7 +12,11 @@
 #include "Utils.h"
 #include "kodi/tools/StringUtils.h"
 
+#include <cctype>
 #include <ctime>
+#include <iomanip>
+#include <ios>
+#include <sstream>
 
 namespace
 {
@@ -43,7 +47,7 @@ ADDON_STATUS PlutotvData::Create()
 ADDON_STATUS PlutotvData::SetSetting(const std::string& settingName,
                                      const kodi::addon::CSettingValue& settingValue)
 {
-  return ADDON_STATUS_OK;
+  return ADDON_STATUS_NEED_RESTART;
 }
 
 PVR_ERROR PlutotvData::GetCapabilities(kodi::addon::PVRCapabilities& capabilities)
@@ -66,6 +70,32 @@ PVR_ERROR PlutotvData::GetBackendVersion(std::string& version)
   return PVR_ERROR_NO_ERROR;
 }
 
+namespace
+{
+// http://stackoverflow.com/a/17708801
+const std::string UrlEncode(const std::string& value)
+{
+  std::ostringstream escaped;
+  escaped.fill('0');
+  escaped << std::hex;
+
+  for (auto c : value)
+  {
+    // Keep alphanumeric and other accepted characters intact
+    if (std::isalnum(c) || c == '-' || c == '_' || c == '.' || c == '~')
+    {
+      escaped << c;
+      continue;
+    }
+
+    // Any other characters are percent-encoded
+    escaped << '%' << std::setw(2) << int(static_cast<unsigned char>(c));
+  }
+
+  return escaped.str();
+}
+} // unnamed namespace
+
 void PlutotvData::SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty>& properties,
                                       const std::string& url,
                                       bool realtime)
@@ -76,10 +106,17 @@ void PlutotvData::SetStreamProperties(std::vector<kodi::addon::PVRStreamProperty
   properties.emplace_back(PVR_STREAM_PROPERTY_INPUTSTREAM, "inputstream.adaptive");
   properties.emplace_back(PVR_STREAM_PROPERTY_ISREALTIMESTREAM, realtime ? "true" : "false");
   // HLS
-  kodi::Log(ADDON_LOG_DEBUG, "[PLAY STREAM] hls");
-  properties.emplace_back("inputstream.adaptive.manifest_type", "hls");
   properties.emplace_back(PVR_STREAM_PROPERTY_MIMETYPE, "application/x-mpegURL");
-  properties.emplace_back("inputstream.adaptive.manifest_update_parameter", "full");
+
+  const std::string encodedUserAgent{UrlEncode(PLUTOTV_USER_AGENT)};
+  properties.emplace_back("inputstream.adaptive.manifest_headers",
+                          "User-Agent=" + encodedUserAgent);
+  properties.emplace_back("inputstream.adaptive.stream_headers",
+                          "User-Agent=" + encodedUserAgent);
+
+  if (GetSettingsWorkaroundBrokenStreams())
+    properties.emplace_back("inputstream.adaptive.manifest_config",
+                            "{\"hls_ignore_endlist\":true,\"hls_fix_mediasequence\":true,\"hls_fix_discsequence\":true}");
 }
 
 bool PlutotvData::LoadChannelsData()
@@ -113,7 +150,8 @@ bool PlutotvData::LoadChannelsData()
   kodi::Log(ADDON_LOG_DEBUG, "[channels] iterate channels");
   kodi::Log(ADDON_LOG_DEBUG, "[channels] size: %i;", channelsDoc["result"].Size());
 
-  int i = 0;
+  // Use configured start channel number to populate the channel list
+  int i = GetSettingsStartChannel();
   for (const auto& channel : channelsDoc["result"].GetArray())
   {
     /**
@@ -153,9 +191,9 @@ bool PlutotvData::LoadChannelsData()
       }}, */
 
     const std::string plutotvid = channel["_id"].GetString();
-    ++i;
+
     PlutotvChannel plutotv_channel;
-    plutotv_channel.iChannelNumber = i; // position
+    plutotv_channel.iChannelNumber = i++; // position
     kodi::Log(ADDON_LOG_DEBUG, "[channel] channelnr(pos): %i;", plutotv_channel.iChannelNumber);
 
     plutotv_channel.plutotvID = plutotvid;
@@ -170,17 +208,25 @@ bool PlutotvData::LoadChannelsData()
     kodi::Log(ADDON_LOG_DEBUG, "[channel] name: %s;", plutotv_channel.strChannelName.c_str());
 
     std::string logo;
-    if (channel.HasMember("logo"))
+    if (GetSettingsColoredChannelLogos())
+    {
+      if (channel.HasMember("colorLogoPNG"))
+        logo = channel["colorLogoPNG"]["path"].GetString();
+    }
+    else
+    {
+      if (channel.HasMember("solidLogoPNG"))
+        logo = channel["solidLogoPNG"]["path"].GetString();
+    }
+    // fallback, should always work
+    if (logo.empty() && channel.HasMember("logo"))
     {
       logo = channel["logo"]["path"].GetString();
-    }
-    else if (channel.HasMember("colorLogoPNG"))
-    {
-      logo = channel["colorLogoPNG"]["path"].GetString();
+      kodi::Log(ADDON_LOG_DEBUG, "[channel] logo (fallback): %s;", logo.c_str());
     }
 
     plutotv_channel.strIconPath = logo;
-    kodi::Log(ADDON_LOG_DEBUG, "[channel] logo: %s;", plutotv_channel.strIconPath.c_str());
+    kodi::Log(ADDON_LOG_DEBUG, "[channel] iconpath: %s;", plutotv_channel.strIconPath.c_str());
 
     if (channel.HasMember("stitched") && channel["stitched"].HasMember("urls") &&
         channel["stitched"]["urls"].Size() > 0)
@@ -262,6 +308,21 @@ std::string PlutotvData::GetSettingsUUID(const std::string& setting)
   return uuid;
 }
 
+int PlutotvData::GetSettingsStartChannel() const
+{
+  return kodi::addon::GetSettingInt("start_channelnum", 1);
+}
+
+bool PlutotvData::GetSettingsColoredChannelLogos() const
+{
+  return kodi::addon::GetSettingBoolean("colored_channel_logos", true);
+}
+
+bool PlutotvData::GetSettingsWorkaroundBrokenStreams() const
+{
+  return kodi::addon::GetSettingBoolean("workaround_broken_streams", true);
+}
+
 std::string PlutotvData::GetChannelStreamURL(int uniqueId)
 {
   LoadChannelsData();
